# Imp rt the modules
import sys
import numpy as np
from netCDF4 import Dataset
from scipy import interpolate as interp
from scipy import special as sp
#from pyproj import Transformer as tr
#from pyproj import CRS

def GetJK(lat,lon,dim):
    fl = '../database/NCM_SpatialGrid.nc'
    nc = Dataset(fl,'r')
    LAv = nc.variables['Latitude vector'][:]
    LOv = nc.variables['Longitude vector'][:]
    jg = nc.variables['Index x grid'][:]
    kg = nc.variables['Index y grid'][:]

    f1 = interp.interp2d(LOv,LAv,np.transpose(jg),kind='linear')
    f2 = interp.interp2d(LOv,LAv,np.transpose(kg),kind='linear')
    J = np.zeros(np.shape(lat),dtype=np.float64)
    K = np.zeros(np.shape(lat),dtype=np.float64)
    if dim == 1:
        nLocs = np.size(lat)
        for l in range(0,nLocs):
            J[l] = f1(lon[l],lat[l])-1
            K[l] = f2(lon[l],lat[l])-1
    else:
        nx,ny = np.shape(lon)
        for x in range(0,nx):
            for y in range(0,ny):
                J[x,y] = f1(lon[x,y],lat[x,y])-1
                K[x,y] = f2(lon[x,y],lat[x,y])-1

    # There appears to be a difference in the projection algorithm between matlab and python. To maintain
    # consistency, I decide to interpolate the spatial index grid which was produced using matlab's
    # algorithms.
    #X1 = -2600000
    #Y1 = 0
    #X2 = 2300000
    #Y2 = 3200000
    #nx = 4900
    #ny = 3200

    #crs_4326 = CRS.from_epsg(4326)
    #crs = CRS.from_proj4("+proj=tmerc +lat_1=30 +lat_2=45 +lat_0=24 +lon_0=-95 +k_0=0.9996 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0")

    #projfwd = tr.from_crs(crs_4326, crs)
    #X,Y = projfwd.transform(lat,lon)
    #J = (ny - 1)*(Y - Y1)/(Y2 - Y1);
    #K = (nx - 1)*(X - X1)/(X2 - X1);

    return J,K

def loadPar(fl,x1,x2,y1,y2):

    nc = Dataset(fl,'r')
    e = nc.variables['Surface Elevation'][x1:x2, y1:y2]
    MohoTemp = nc.variables['Moho Temperature'][x1:x2, y1:y2]
    SurfaceTemp = nc.variables['Surface Temperature'][x1:x2, y1:y2]
    SurfaceTempAve = nc.variables['Surface Temperature Average'][x1:x2, y1:y2]
    SurfaceTempGradient = nc.variables['Surface Temperature Gradient'][x1:x2, y1:y2]
    Conductivity = nc.variables['Conductivity'][x1:x2, y1:y2]
    LIDe = nc.variables['Moho Elevation'][x1:x2, y1:y2]
    Top2 = nc.variables['Ocean Top Elevation Smooth'][x1:x2, y1:y2]
    Age2 = nc.variables['Ocean Age'][x1:x2, y1:y2]
    AgeH = nc.variables['Ocean Age Mask'][x1:x2, y1:y2]
    TransDepth = nc.variables['Transition Depth'][x1:x2, y1:y2]
    TransDepthType = nc.variables['Transition Depth Type'][x1:x2, y1:y2]
    TransDepthOcean = nc.variables['Transition Depth Ocean'][x1:x2, y1:y2]
    TransDepthTypeOcean = nc.variables['Transition Depth Type Ocean'][x1:x2, y1:y2]

    l1 = 9+((80-9)/80)*Age2;
    l2 = 80*np.ones((np.shape(Age2)),dtype=np.float64);
    AgeCorr = 1/((1/l1)**10 + (1/l2)**10);
    AgeCorr = AgeCorr**(1/10);

    return e,Top2,AgeCorr,AgeH,Conductivity,SurfaceTemp,SurfaceTempAve,SurfaceTempGradient, \
        MohoTemp,LIDe,TransDepth,TransDepthType,TransDepthOcean,TransDepthTypeOcean

def CalcTemp(lat,lon,Depth):
    dim = np.size(np.shape(Depth))
    j,k = GetJK(lat,lon,dim)

    x1 = int(np.fmax(0,np.floor(np.min(k))-5))
    x2 = int(np.fmin(4901,np.ceil(np.max(k))+5))+1
    y1 = int(np.fmax(0,np.floor(np.min(j))-5))
    y2 = int(np.fmin(3201,np.ceil(np.max(j))+5))+1

    fl = '../database/NCM_TemperatureGrids.nc'
    e,Top2,AgeCorr,AgeH,Conductivity,SurfaceTemp,SurfaceTempAve,SurfaceTempGradient, \
        MohoTemp,LIDe,TransDepth,TransDepthType,TransDepthOcean,TransDepthTypeOcean = loadPar(fl,x1,x2,y1,y2)

    xv = np.arange(x1,x2,1)
    yv = np.arange(y1,y2,1)

    f1 = interp.interp2d(xv,yv,np.transpose(e),kind='cubic')
    f2 = interp.interp2d(xv,yv,np.transpose(SurfaceTemp),kind='cubic')
    f3 = interp.interp2d(xv,yv,np.transpose(Conductivity),kind='cubic')
    f4 = interp.interp2d(xv,yv,np.transpose(SurfaceTempAve),kind='cubic')
    f5 = interp.interp2d(xv,yv,np.transpose(SurfaceTempGradient),kind='cubic')
    f6 = interp.interp2d(xv,yv,np.transpose(MohoTemp),kind='cubic')
    f7 = interp.interp2d(xv,yv,np.transpose(LIDe),kind='cubic')
    f8 = interp.interp2d(xv,yv,np.transpose(Top2),kind='cubic')
    f9 = interp.interp2d(xv,yv,np.transpose(AgeCorr),kind='cubic')
    f10 = interp.interp2d(xv,yv,np.transpose(AgeH),kind='cubic')
    #f1 = interp.RectBivariateSpline(xv,yv,e)
    #f2 = interp.RectBivariateSpline(xv,yv,SurfaceTemp)
    #f3 = interp.RectBivariateSpline(xv,yv,Conductivity)
    #f4 = interp.RectBivariateSpline(xv,yv,SurfaceTempAve)
    #f5 = interp.RectBivariateSpline(xv,yv,SurfaceTempGradient)
    #f6 = interp.RectBivariateSpline(xv,yv,MohoTemp)
    #f7 = interp.RectBivariateSpline(xv,yv,LIDe)
    #f8 = interp.RectBivariateSpline(xv,yv,Top2)
    #f9 = interp.RectBivariateSpline(xv,yv,AgeCorr)
    #f10 = interp.RectBivariateSpline(xv,yv,AgeH)

    ei = np.zeros(np.shape(j),dtype=np.float64)
    Ts = np.zeros(np.shape(j),dtype=np.float64)
    ks = np.zeros(np.shape(j),dtype=np.float64)
    TsAve = np.zeros(np.shape(j),dtype=np.float64)
    dTs = np.zeros(np.shape(j),dtype=np.float64)
    Tm = np.zeros(np.shape(j),dtype=np.float64)
    Md = np.zeros(np.shape(j),dtype=np.float64)
    DepthOceanPlate = np.zeros(np.shape(j),dtype=np.float64)
    AC = np.zeros(np.shape(j),dtype=np.float64)
    AH = np.zeros(np.shape(j),dtype=np.float64)
    if dim == 1:
        nLocs = np.size(j)
        for l in range(0,nLocs):
            ei[l] = f1(k[l],j[l])
            Ts[l] = f2(k[l],j[l])
            ks[l] = f3(k[l],j[l])
            TsAve[l] = f4(k[l],j[l])
            dTs[l] = f5(k[l],j[l])
            Tm[l] = f6(k[l],j[l])
            Md[l] = ei[l] - f7(k[l],j[l])
            DepthOceanPlate[l] = ei[l] - f8(k[l],j[l])
            AC[l] = f9(k[l],j[l])
            AH[l] = f10(k[l],j[l])
    else:
        ny,nx = np.shape(j)
        for l in range(0,ny):
            for m in range(0,nx):
                ei[l,m] = f1(k[l,m],j[l,m])
                Ts[l,m] = f2(k[l,m],j[l,m])
                ks[l,m] = f3(k[l,m],j[l,m])
                TsAve[l,m] = f4(k[l,m],j[l,m])
                dTs[l,m] = f5(k[l,m],j[l,m])
                Tm[l,m] = f6(k[l,m],j[l,m])
                Md[l,m] = ei[l,m] - f7(k[l,m],j[l,m])
                DepthOceanPlate[l,m] = ei[l,m] - f8(k[l,m],j[l,m])
                AC[l,m] = f9(k[l,m],j[l,m])
                AH[l,m] = f10(k[l,m],j[l,m])

    hr = 0.3*Md
    km = 3.6
    # Stacey (1992), p 287
    kum = 4.5 
    kr = Md/np.log(km/ks)

    secPerYear = 365.24*24*60*60

    # From Stacey (2008), Appendix G3 from ~200 to ~400 km depth
    LithMantleGrad2 = 0.0004    
    TMantle = LithMantleGrad2*(Depth - 410e3) + 1579.85

    # Turcotte and Schubert, p. 164
    TM = 1300-273.15 
    kappa = 1e-6
    AgeSeconds = AC*1e6*secPerYear

    # Correction for higher surface temperature of DTS over the last 10 ka
    P = 10e3*secPerYear
    K = 1e-6
    CharLength = np.sqrt(K*P/np.pi)
    DTS = Ts - TsAve

    # Correct for approaching the solidus
    # Basalt solidus from Turcotte and Schubert, Figure 4-8
    SolidusGradient = 200/100e3
    TSolidus = 1100 + Depth*SolidusGradient

    TMS = 1/((1/TMantle)**30 + (1/TSolidus)**30)**(1/30) + 273.15

    # With increasing conductivity and decreasing heat production
    Hr = 1/(1/hr + 1./kr)
    pHs_o_k = (Tm*(1 + Md/kr) - dTs*Md -  TsAve*(1 + Md/kr))/(Hr**2 - Hr*Md - Hr**2*np.exp(-Md/Hr))
    c1 = dTs + TsAve/kr - pHs_o_k*Hr
    c2 = TsAve + pHs_o_k*Hr**2

    if dim == 2:
        T = np.zeros(np.shape(Depth),dtype=np.float64)
        f1 = interp.interp2d(xv,yv,np.transpose(TransDepth),kind='cubic')
        f2 = interp.interp2d(xv,yv,np.transpose(TransDepthType),kind='linear')
        f3 = interp.interp2d(xv,yv,np.transpose(TransDepthOcean),kind='cubic')
        f4 = interp.interp2d(xv,yv,np.transpose(TransDepthTypeOcean),kind='linear')
        trd = np.zeros(np.shape(j),dtype=np.float64)
        trdt = np.zeros(np.shape(j),dtype=np.float64)
        trdo = np.zeros(np.shape(j),dtype=np.float64)
        trdto = np.zeros(np.shape(j),dtype=np.float64)
        for l in range(0,ny):
            for m in range(0,nx):
                trd[l,m] = f1(k[l,m],j[l,m])
                trdt[l,m] = np.round(f2(k[l,m],j[l,m]))
                trdo[l,m] = f3(k[l,m],j[l,m])
                trdto[l,m] = np.round(f4(k[l,m],j[l,m]))
        l = np.nonzero(trdt == 3)
        trdt[l] = 2
        l = np.nonzero(trdto == 3)
        trdto[l] = 1

        T1 = (c1*Depth + c2 - pHs_o_k*Hr**2*np.exp(-Depth/Hr))/(1 + Depth/kr)

        # Correct T1 for very low gradients
        l = np.nonzero((trdt == 2) & (Depth-trd > 0))
        Ttr = (c1[l]*trd[l] + c2[l] - pHs_o_k[l]*Hr[l]**2*np.exp(-trd[l]/Hr[l]))/ \
            (1 + trd[l]/kr[l])
        T1[l] = Ttr + 2.0*SolidusGradient*(Depth[l] - trd[l])

        l = np.nonzero((trdt == 2) & (Depth-Md > 0))
        dTdZmSub = km*2.0*SolidusGradient/kum
        Ttr = (c1[l]*trd[l] + c2[l] - pHs_o_k[l]*Hr[l]**2.*np.exp(-trd[l]/Hr[l]))/ \
            (1 + trd[l]/kr[l])
        Tm[l] = Ttr + 2.0*SolidusGradient*(Md[l] - trd[l])
        T1[l] = dTdZmSub*(Depth[l] - Md[l]) + Tm[l]

        # Modify normal geotherm for depth greater than Moho, Md
        l = np.nonzero((trdt == 0) & (Depth-Md > 0))
        dTdZm = -(1/kr[l])*(c1[l]*Md[l] + c2[l] - pHs_o_k[l]*Hr[l]**2* \
            np.exp(-Md[l]/Hr[l]))/(1 + Md[l]/kr[l])**2 + \
            (c1[l] + pHs_o_k[l]*Hr[l]*np.exp(-Md[l]/Hr[l]))/(1 + Md[l]/kr[l])
        dTdZmSub = km*dTdZm/kum
        T1[l] = dTdZmSub*(Depth[l] - Md[l]) + Tm[l]

        # add to TMS
        T1 = T1 + 273.15
        T1 = 1/((1/T1)**10 + (1/TMS)**10)**(1/10)
        T1 = T1 - 273.15
        T2 = np.copy(T1)

        l = np.nonzero(Depth > DepthOceanPlate)
        if np.size(l) > 0:
            TsO = CalcTemp(lat,lon,DepthOceanPlate)
            phi = 2*np.sqrt(kappa*AgeSeconds)
            dt = (np.fmax(TM*np.ones(np.shape(TsO)),TsO) - TsO)
            T2[l] = dt[l]*sp.erf((Depth[l] - DepthOceanPlate[l])/phi[l]) + TsO[l]
            m = np.nonzero((Depth > DepthOceanPlate) & ((Depth > trdo) & ((trdto == 1) | (trdto == 2))))
            if np.size(m) > 0:
                n = np.nonzero(((Depth > DepthOceanPlate) & ((Depth > trdo) & \
                    ((trdto == 1) | (trdto == 2)))) & trdto == 1)
                z = (trdo[n] - DepthOceanPlate[n])/phi[n]
                dTdZ = dt[n]*(2/np.sqrt(pi))*np.exp(-z**2)*(1/phi[n])
                Ttr = dt[n]*sp.erf(z) + TsO[n]
                T2[n] = Ttr + dTdZ*(Depth[n] - trdo[n])

                n = np.nonzero(((Depth > DepthOceanPlate) & ((Depth > trdo) & \
                    ((trdto == 1) | (trdto == 2)))) & trdto == 2)
                Ttr = dt[n]*sp.erf((trdo[n] - DepthOceanPlate[n])/phi[n]) + TsO[n]
                T2[n] = Ttr + LithMantleGrad2*(Depth[n] - trdo[n])
            T2 = T2 + 273.15
            T2 = 1/((1/T2)**10 + (1/TMS)**10)**(1/10)
            T2 = T2 - 273.15

        T = AH*T2 + (1-AH)*T1
        DT = DTS*np.exp(-Depth/CharLength)
        T = T + DT
        return T
    else:
        nDepth = np.size(Depth)
        T = np.zeros((nLocs,nDepth),dtype=np.float64)
        for l in range(0,nLocs):
            T1 = np.zeros([nDepth],dtype=np.float64)
            T2 = np.zeros([nDepth],dtype=np.float64)

            # With increasing conductivity and decreasing heat production
            T1 = (c1[l]*Depth + c2[l] - pHs_o_k[l]*Hr[l]**2*np.exp(-Depth/Hr[l]))/(1 + Depth/kr[l])
            dTdZ = np.gradient(T1)/(Depth[1]-Depth[0])
            j = np.nonzero(dTdZ < 2.0*SolidusGradient)[0]
            if np.size(j) > 0:
                if j[0] > 0:
                    J = np.arange(j[0]-1,nDepth,1)
                    T1[J] = T1[j[0]-1] + 2.0*SolidusGradient*(Depth[J] - Depth[j[0]-1])
    
            # Steepen gradient below the Moho
            if np.size(j) == 0:
                dTdZm = -(1/kr[l])*(c1[l]*Md[l] + c2[l] - \
                    pHs_o_k[l]*Hr[l]**2*np.exp(-Md[l]/Hr[l]))/(1 + Md[l]/kr[l])**2 + \
                    (c1[l] + pHs_o_k[l]*Hr[l]*np.exp(-Md[l]/Hr[l]))/(1 + Md[l]/kr[l])
                dTdZmSub = km*dTdZm/kum
                j = np.nonzero(Depth >= Md[l])[0]
                T1[j] = Tm[l] + dTdZmSub*(Depth[j] - Md[l])
            else:
                if Depth[j[0]] > Md[l]:
                    dTdZm = -(1/kr[l])*(c1[l]*Md[l] + c2[l] - \
                        pHs_o_k[l]*Hr[l]**2*np.exp(-Md[l]/Hr[l]))/(1 + Md[l]/kr[l])**2 + \
                        (c1[l] + pHs_o_k[l]*Hr[l]*np.exp(-Md[l]/Hr[l]))/(1 + Md[l]/kr[l])
                    dTdZmSub = km*dTdZm/kum
    
                    j = np.nonzero(Depth >= Md[l])[0]
                    T1[j] = Tm[l] + dTdZmSub*(Depth[j] - Md[l])
                else:
                    dTdZmSub = km*2.0*SolidusGradient/kum
                    j = np.nonzero(Depth >= Md[l])[0]
                    f = interp.interp1d(Depth,T1,kind='linear')
                    Tm[l] = f(Md[l])
                    T1[j] = Tm[l] + dTdZmSub*(Depth[j] - Md[l])

            T1 = T1 + 273.15
            T1 = 1./((1./T1)**10 + (1./TMS)**10)**(1/10)
            T1 = T1 - 273.15
            T2 = np.copy(T1)
            
            j = np.nonzero(Depth > DepthOceanPlate[l])[0]
            if np.size(j) > 0:
                TsO = T1[j[0]]
                T2[j] = (np.fmax(TM,TsO) - TsO)*sp.erf((Depth[j] - DepthOceanPlate[l])/ \
                    (2.0*np.sqrt(kappa*AgeSeconds[l]))) + TsO
                dTdD = np.gradient(T2)/np.gradient(Depth)
                k = np.nonzero(T2 > TMS - 273.15)[0]
                if np.size(k) > 0:
                    K = np.arange(k[0],nDepth,1)
                    T2[K] = T2[k[0]] + dTdD[k[0]]*(Depth[K] - Depth[k[0]])
                else:
                    k = np.nonzero((dTdD < LithMantleGrad2) & (Depth > DepthOceanPlate[l]))[0]
                    if np.size(k) > 0:
                        if k[0] > 0:
                            K = np.arange(k[0]-1,nDepth,1)
                            T2[K] = T2[k[0]-1] + LithMantleGrad2*(Depth[K] - Depth[k[0]-1])
                        else:
                            K = np.arange(k[0],nDepth,1)
                            T2[K] = T2[k[0]] + LithMantleGrad2*(Depth[K] - Depth[k[0]])
    
            TT = AH[l]*T2 + (1-AH[l])*T1

            # Correct for higher surface temperature of DTS over the last 100 ka
            DT = DTS[l]*np.exp(-Depth/CharLength)
            TT = TT + DT

            T[l,:] = np.transpose(TT)

        return T
