# Import the modules
import sys
import TherMod as tm
import numpy as np
import time

t1 = time.time()
if len(sys.argv) > 1:
    if sys.argv[1] == "-h":
        print('TherMod -- Program to calculate temperature profile within the National Crustal Model')
        print('')
        print('        Written by Oliver Boyd, v1. 19/09/27')
        print('')
        print('        INPUTS')
        print('        Command line options')
        print('        -h Help about this program.')
        print('')
        print('        -f file - File containing depth vector along the first line and locations of interest along')
        print('           the remaining. For example, to get a profile at a series of locations')
        print('           MinDepth,DepthIncrement,MaxDepth')
        print('           Lat1,Lon1')
        print('           Lat2,Lon2')
        print('           LatN,LonN')
        print('        OR')
        print('           To get a map at a given depth')
        print('           Depth')
        print('           LatMin,LatIncrement,LatMax,LonMin,LonIncrement,LonMax')
        print('')
        print('        For a profile:')
        print('        -l lat,lon - latitude and longitude of point of interest')
        print('        -d MinDepth,DepthIncrement,MaxDepth - depth vector in meters')
        print('        OR')
        print('        a map:')
        print('        -l LatMin,LatIncrement,LatMax,LonMin,LonIncrement,LonMax - latitude and longitude')
        print('           grid of interest')
        print('        -d Depth - depth of map')
        print('')
        print('        OUTPUT: Temperature in celcius at the locations requested.')
        print('')
        print('     Examples:');
        print('     Temperature profile at a latitude and longitude of 36, -118 from the surface to 20 km depth in')
        print('     increments of 100 m.')
        print('     > python TherModWrapper.py -l 36,-118 -d 0,1e2,20e3')
        print('')
        print('     Temperature map at a depth of 10 km on a 0.02 degree grid from 30 to 40 degrees latitude')
        print('     and -125 to -115 degrees longitude')
        print('     > python TherModWrapper.py -l 30,0.02,35,-125,0.02,-120 -d 1e3')
        print('')
        sys.exit()

if len(sys.argv) > 1:
    for j in range(1,len(sys.argv),2):
        if sys.argv[j] == "-l":
            entries = sys.argv[j+1].split(",")
            if np.size(entries) == 2:
                nLocs = 1
                lat = np.zeros((nLocs),dtype=np.float64)
                lon = np.zeros((nLocs),dtype=np.float64)
                lat[0] = entries[0]
                lon[0] = entries[1]
                profile = 1
            else:
                latm = np.zeros((1),dtype=np.float64)
                lati = np.zeros((1),dtype=np.float64)
                latM = np.zeros((1),dtype=np.float64)
                lonm = np.zeros((1),dtype=np.float64)
                loni = np.zeros((1),dtype=np.float64)
                lonM = np.zeros((1),dtype=np.float64)
                latm[0] = entries[0]
                lati[0] = entries[1]
                latM[0] = entries[2]
                lonm[0] = entries[3]
                loni[0] = entries[4]
                lonM[0] = entries[5]
                [lon,lat] = np.meshgrid(np.linspace(lonm,lonM, \
                    np.round((lonM-lonm)/loni)+1),np.linspace(latm, \
                    latM,np.round((latM-latm)/lati)+1))
                nLocs = np.shape(lon)
                profile = 0
        if sys.argv[j] == "-d":
            entries = sys.argv[j+1].split(",")
            if np.size(entries) == 3:
                depmin = np.zeros((1),dtype=np.float64)
                depinc = np.zeros((1),dtype=np.float64)
                depmax = np.zeros((1),dtype=np.float64)
                depmin[0] = entries[0]
                depinc[0] = entries[1]
                depmax[0] = entries[2]
            else:
                depval = np.zeros((1),dtype=np.float64)
                depval[0] = entries[0]
        if sys.argv[j] == "-f":
            fl = sys.argv[j+1]
            print('Reading {0:s}'.format(fl))
            f = open(fl,"r")
            if f.mode == "r":
                nLocs = 0
                ln = 0
                for line in f:
                    line = line.strip()
                    columns = line.split(",")
                    if ln == 0:
                        if np.size(columns) == 3:
                            depmin = np.zeros((1),dtype=np.float64)
                            depinc = np.zeros((1),dtype=np.float64)
                            depmax = np.zeros((1),dtype=np.float64)
                            depmin[0] = columns[0]
                            depinc[0] = columns[1]
                            depmax[0] = columns[2]
                            profile = 1
                        else:
                            depval[0] = columns[0]
                            profile = 0
                    else:
                        nLocs = nLocs + 1
                    ln = ln + 1
            f.close()
            if profile == 1:
                lat = np.zeros((nLocs),dtype=np.float64)
                lon = np.zeros((nLocs),dtype=np.float64)
            else:
                latm = np.zeros((1),dtype=np.float64)
                lati = np.zeros((1),dtype=np.float64)
                latM = np.zeros((1),dtype=np.float64)
                lonm = np.zeros((1),dtype=np.float64)
                loni = np.zeros((1),dtype=np.float64)
                lonM = np.zeros((1),dtype=np.float64)
            f = open(fl,"r")
            if f.mode == "r":
                ln = 0
                jT = 0
                for line in f:
                    line = line.strip()
                    columns = line.split(",")
                    if ln > 0:
                        if profile == 1:
                            lat[jT] = columns[0]
                            lon[jT] = columns[1]
                            jT = jT + 1
                        else:
                            latm[0] = entries[0]
                            lati[0] = entries[1]
                            latM[0] = entries[2]
                            lonm[0] = entries[3]
                            loni[0] = entries[4]
                            lonM[0] = entries[5]
                            [lon,lat] = np.meshgrid(np.linspace(lonm,lonM, \
                                np.round((lonM-lonm)/loni)+1),np.linspace(latm, \
                                latM,np.round((latM-latm)/lati)+1))
                    ln = ln + 1
            f.close()

if profile == 0:
    Depth = depval*np.ones(np.shape(lon))
else:
    Depth = np.linspace(depmin,depmax,np.round((depmax-depmin)/depinc)+1)
    nDepth = np.size(Depth)
    Depth = np.reshape(Depth,nDepth)

T = tm.CalcTemp(lat,lon,Depth)

print('Saving temperatures in Temps.csv')
if profile == 1:
    np.savetxt("Temps.csv",np.transpose(T), delimiter=",")
else:
    np.savetxt("Temps.csv",T, delimiter=",")

np.save('DepthVec.npy',Depth)
np.save('Temps.npy',T)

elapsed_time = time.time()-t1
print('Elapsed time: ',elapsed_time)
sys.exit()
